/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function() {

    /**
     *  Object that holds a current random data piece
     *  @private
     */
    var data = {
        name: null,
        rawData: null,
        next: function() {
            var guid = function() {
                return 'xxxxxxxx'.replace(/[xy]/g, function(c) {
                    var r = Math.random() * 16 | 0,
                        v = (c == 'x' ? r : (r&0x3 | 0x8));
                    return v.toString(16);
                });
            };

            this.name = guid();
            this.rawData = guid();
            return this;
        }
    };


    /**
     *  Displays error message in the popup
     *  @private
     *  @param {string} message - Error message to display in the popup
     */
    function displayErrorPopup(message) {
        document.getElementById('error-popup').querySelector('.ui-popup-content').innerText = message;
        tau.openPopup('error-popup');
    }


    /**
     *  Gets list of aliases of the saved data
     *  @private
     */
    function getDataAliases() {
        return tizen.keymanager.getDataAliasList();
    }


    /**
     *  Gets raw data by the alias given
     *  @private
     *  @param {Object} alias - Alias of the data piece to get
     */
    function getRawData(alias) {
        var data;
        try {
            data = tizen.keymanager.getData(alias);
        } catch (e) {
            if (e.name === "NotFoundError") {
                data = "Data inaccessible";
            }
        }
        return data;
    }


    /**
     *  Saves current random data and returns to the main page
     *  @private
     */
    function saveDataPiece() {
        var name = this.querySelector('.data.name > input').value,
            rawData = this.querySelector('.data.raw-data > input').value;

        tizen.keymanager.saveData(name, rawData, null, function() {
            tau.back();
            data.next();
        }, function(e) {
            displayErrorPopup('Cannot save the data: ' + e.message);
        });
    }


    /**
     *  Removes data given with the alias and returns to the main page
     *  @private
     *  @param {Object} alias - Alias of the data piece to remove
     */
    function removeDataPiece(alias) {
        try {
            tizen.keymanager.removeData(alias);
            tau.back();
        } catch (e) {
            displayErrorPopup('Cannot remove the data: ' + e.message);
        }
    }


    /**
     *  Displays data in the other page given with the alias
     *  @private
     *  @param {Object} alias - Alias of the data piece to display
     */
    function displayDataPiece(alias) {
        var name = alias.name,
            rawData = getRawData(alias);

        this.querySelector('.data.name > span').innerHTML = name;
        this.querySelector('.data.raw-data > span').innerHTML = rawData;
        this.querySelector('#btn-remove-data').onclick = removeDataPiece.bind(this, alias);
        this.querySelector('#btn-set-permission').onclick = function() {
            document.getElementById('page-set-permission').querySelector('#alias-name > span').innerHTML = alias.name;
            tau.changePage('page-set-permission');
        };
        tau.changePage('page-display-data');
    }


    /**
     *  Sets a permission to the file for a given package
     *  @private
     */
    function setPermission() {
        var dataName = this.querySelector('.data.name > span').innerText,
            accessorPackageID = this.querySelector('#get-package-id > input'),
            permissionType = this.querySelector('input[name="permission-type"]:checked');

        if (!permissionType) {
            displayErrorPopup('Please choose a permission type.');
            return;
        }

        tizen.keymanager.setPermission({ 'name': dataName }, accessorPackageID.value, permissionType.value, function() {
            tau.changePage('page-main');
        }, function() {
            displayErrorPopup('Cannot set permission for that data.');
        });
    }


    /**
     *  Populates list view with data retrieved from Key Manager
     *  @private
     */
    function listDataPieces() {
        var aliases = getDataAliases(),
            displayDataPage = document.getElementById('page-display-data'),
            dataList = document.getElementById('data-list');

        // Cleans up the list
        while (dataList.firstChild) {
            dataList.removeChild(dataList.firstChild);
        }

        if (aliases.length > 0) {
            // For each alias a list item is created
            for (var i = 0; i < aliases.length; ++i) {
                var li = document.createElement('li'),
                    a = document.createElement('a'),
                    text = document.createTextNode(aliases[i].name);

                a.appendChild(text);
                a.href='#';
                a.onclick = displayDataPiece.bind(displayDataPage, aliases[i]);
                li.appendChild(a);
                dataList.appendChild(li);
            }
        } else {
            // Else the list is populated with one non-interactive item "No data"
            var li = document.createElement('li'),
                i = document.createElement('i'),
                text = document.createTextNode('No data');

            i.appendChild(text);
            li.appendChild(i);
            dataList.appendChild(li);
        }
    }


    // Page selectors
    var mainPage = document.getElementById('page-main'),
        addDataPage = document.getElementById('page-add-data'),
        setPermissionPage = document.getElementById('page-set-permission'),
        errorPopupPage = document.getElementById('error-popup');


    // Configures animations
    tau.defaults.pageTransition = 'slideup';
    tau.defaults.popupTransition = 'slideup';


    window.addEventListener('tizenhwkey', function(ev) {
        if (ev.keyName === 'back') {
            var page = document.getElementsByClassName('ui-page-active')[0], pageid = page ? page.id : '';
            if (pageid === 'page-main') {
                try {
                  tizen.application.getCurrentApplication().exit();
                } catch (ignore) {
                }
            } else {
                window.history.back();
            }
        }
    });


    /**
     *  Registers a listener for the main page
     */
    mainPage.addEventListener('pagebeforecreate', function() {
        // Initializes random data
        data.next();
    });


    /**
     *  Registers a listener for the main page
     */
    mainPage.addEventListener('pagebeforeshow', function() {
        var dataListContainer = this.querySelector('.ui-content'),
            dataPopup = document.getElementById('popup-data');

        // Registers rotary event
        var SCROLL_STEP = 30;
        this.handleRotaryEvent = function (e) {
            if (e.detail.direction === 'CW') {
                dataListContainer.scrollTop += SCROLL_STEP;
            } else if (e.detail.direction === 'CCW') {
                dataListContainer.scrollTop -= SCROLL_STEP;
            }
        }

        document.addEventListener('rotarydetent', this.handleRotaryEvent, false);

        // Changes page on click
        this.querySelector('#btn-add-data').onclick = function() {
            tau.changePage('page-add-data');
        };

        // Initializes list of saved pieces
        listDataPieces.apply(this);
    });


    /**
     *  Registers a listener for the main page
     */
    mainPage.addEventListener('pagebeforehide', function() {
        document.removeEventListener('rotarydetent', this.handleRotaryEvent);
    });


    /**
     *  Registers a listener for the add data page
     */
    addDataPage.addEventListener('pagebeforeshow', function() {
        this.querySelector('#btn-save-data').onclick = saveDataPiece.bind(this);

        // Displays current random data
        this.querySelector('.data.name > input').value = data.name;
        this.querySelector('.data.raw-data > input').value = data.rawData;

        this.querySelectorAll('input[type="text"]').forEach(function(inputElement) {
            tau.widget.TextInput(inputElement);
        });
    });


    /**
     *  Registers a listener for the set permission page
     */
    setPermissionPage.addEventListener('pagebeforeshow', function() {
        this.querySelector('#btn-set').onclick = setPermission.bind(this);
        this.querySelector('#get-package-id > input').value = '';

        this.querySelectorAll('input[type="text"]').forEach(function(inputElement) {
            tau.widget.TextInput(inputElement);
        });

        // Unchecks permission type radio buttons
        var permissionRadioList = this.querySelectorAll('#permission-type-list > li');
        for (var i = 0; i < permissionRadioList.length; ++i) {
            permissionRadioList[i].querySelector('input[type="radio"]').checked = false;
        }
    });


    /**
     *  Registers a listener for the error popup page
     */
    errorPopupPage.addEventListener('popupbeforecreate', function() {
        this.querySelector('#btn-ok').onclick = function() {
            tau.closePopup();
        };
    });
})();
